from .base import DoorLoopBase


class DoorLoopPortfolio(DoorLoopBase):
    endpoint_base = '/property-groups'

    def list(self, **kwargs):
        '''
        List all Portfolios
        https://api.doorloop.com/reference/get-property-groups
        '''
        response = self.connector.get(f'{self.endpoint_base}', **kwargs)
        return self.validator(response).validate()

    def retrieve(self, portfolio_id, **kwargs):
        '''
        Retrieve a Portfolio
        https://api.doorloop.com/reference/get-property-group
        '''
        response = self.connector.get(f'{self.endpoint_base}/{portfolio_id}', **kwargs)
        return self.validator(response).validate()



from .base import DoorLoopBase


class DoorLoopAccount(DoorLoopBase):
    endpoint_base = '/accounts'
    # TODO finish setting up account types
    # TODO create function to get verbose account type
    ACCOUNT_TYPES = (
        ('Asset > Other Current Assets', 'ASSET_OTHER_CURRENT_ASSETS'),
        ('Asset > Bank', 'ASSET_BANK'),
    )
    '''
    'ASSET_FIXED_ASSETS',
            'ASSET_OTHER_ASSETS',
            'LIABILITY_ACCOUNTS_PAYABLE',
            'LIABILITY_CREDIT_CARD',
            'LIABILITY_OTHER_CURRENT_LIABILIY',
            'LIABILITY_LONG_TERM_LIABILITY',
            'EQUITY_EQUITY',
            'REVENUE_INCOME',
            'REVENUE_OTHER_INCOME',
            'EXPENSE_EXPENSE',
            'EXPENSE_OTHER_EXPENSE',
            'EXPENSE_COGS'
    '''

    def list(self, **kwargs):
        '''
        List all Accounts
        https://api.doorloop.com/reference/get-accounts
        '''
        response = self.connector.get(f'{self.endpoint_base}', **kwargs)
        return self.validator(response).validate()

    def create(self, data, **kwargs):
        '''
        Create an Account
        https://api.doorloop.com/reference/post-account
        '''
        response = self.connector.post(f'{self.endpoint_base}', data, **kwargs)
        return self.validator(response).validate()

    def retrieve(self, account_id, **kwargs):
        '''
        Retrieve an Account
        https://api.doorloop.com/reference/get-account
        '''
        response = self.connector.get(f'{self.endpoint_base}/{account_id}', **kwargs)
        return self.validator(response).validate()

    def update(self, account_id, data, **kwargs):
        '''
        Update an Account
        https://api.doorloop.com/reference/put-account
        '''
        response = self.connector.put(f'{self.endpoint_base}/{account_id}', data, **kwargs)
        return self.validator(response).validate()

    def delete(self, account_id, **kwargs):
        '''
        Delete an Account
        https://api.doorloop.com/reference/delete-account
        '''
        response = self.connector.delete(f'{self.endpoint_base}/{account_id}', **kwargs)
        return self.validator(response).validate()



import json
import requests
import logging

logger = logging.getLogger('__name__')


class DoorLoopBase:
    endpoint_base = ''

    def __init__(self, connector, validator):
        self.connector = connector
        self.validator = validator

    def pagination(self, func, **kwargs):
        page = 1
        total_pages = 2
        while page < total_pages:
            kwargs['page'] = page
            result = func(**kwargs)
            if result:
                total_pages = result[0].get('total_pages') + 1
            page += 1
            yield result

    def sorting(self, func, **kwargs):
        pass


class DoorLoopConnector:

    def __init__(self, api_key):
        self.api_key = api_key
        self.base_url = f"https://app.doorloop.com/api/"

    def get_url(self, endpoint):
        return self.base_url + endpoint

    def __getattr__(self, name):
        def f(endpoint, data=None, **kwargs):
            # validate function called is a valid request method
            upper_name = name.upper()
            if upper_name not in ('GET', 'POST', 'PUT', 'PATCH', 'HEAD',
                                  'DELETE', 'OPTIONS', 'CONNECT'):
                return name
            # Add Header Data to Request
            if kwargs.get('headers'):
                # if headers passed to function
                if not kwargs['headers'].get('Accept'):
                    kwargs['headers']['Accept'] = 'application/json'
                if not kwargs['headers'].get('Authorization'):
                    kwargs['headers']['Authorization'] = f"Bearer {self.api_key}"
            else:
                # set default headers if not passed
                kwargs['headers'] = {
                    'Accept': 'application/json',
                    'Authorization': f"Bearer {self.api_key}"
                }

            if upper_name in ('POST', 'PUT') and not kwargs['headers'].get('Content-Type'):
                # Add Content-Type header when required by DoorLoop API
                kwargs['headers']['Content-Type'] = 'application/json'

            if data:
                # Convert data to json before sending
                kwargs['data'] = json.dumps(data)

            return DoorLoopConnector.api_call(upper_name, self.get_url(endpoint), **kwargs)

        return f

    @staticmethod
    def api_call(upper_name, api_endpoint, **kwargs):
        def f(method_name, endpoint, **fkwargs):
            response = requests.request(method_name, endpoint, **fkwargs)
            return response
        return f(upper_name, api_endpoint, **kwargs)


class DoorLoopValidator:
    def __init__(self):
        pass

    def __call__(self, response):
        self.response = response
        return self

    def validate(self):
        status_code = self.response.status_code
        try:
            data = self.response.json()
        except Exception as e:
            logger.info(str(e))
            data = {}

        if isinstance(data, dict) and data.get('status') and (
                isinstance(data.get('status'), int) or data.get('status').isdigit()):
            status_code = int(data.get('status'))

        if status_code < 200 or status_code > 299:
            error_message = data.get('message') if data else None
            if not error_message and data and 'base' in data and isinstance(data['base'], list):
                data = ';'.join(data['base'])
            if not error_message and self.response.text:
                data = self.response.text
            logger.error(f'{status_code} Response: {data}')
        return data



from .base import DoorLoopBase


class DoorLoopFile(DoorLoopBase):
    endpoint_base = '/files'

    def list(self, **kwargs):
        '''
        List all Files
        https://api.doorloop.com/reference/get-files
        '''
        response = self.connector.get(f'{self.endpoint_base}', **kwargs)
        return self.validator(response).validate()

    def upload(self, data, **kwargs):
        '''
        Upload a File
        https://api.doorloop.com/reference/post-files
        '''
        # TODO convert file to string / binary file
        response = self.connector.post(f'{self.endpoint_base}', data, **kwargs)
        return self.validator(response).validate()

    def retrieve(self, file_id, **kwargs):
        '''
        Retrieve a File
        https://api.doorloop.com/reference/get-file
        '''
        response = self.connector.get(f'{self.endpoint_base}/{file_id}', **kwargs)
        return self.validator(response).validate()

    def update(self, file_id, data, **kwargs):
        '''
        Update a File
        https://api.doorloop.com/reference/put-file
        '''
        response = self.connector.put(f'{self.endpoint_base}/{file_id}', data, **kwargs)
        return self.validator(response).validate()

    def delete(self, file_id, **kwargs):
        '''
        Delete a File
        https://api.doorloop.com/reference/delete-file
        '''
        response = self.connector.delete(f'{self.endpoint_base}/{file_id}', **kwargs)
        return self.validator(response).validate()

    def download(self, file_id, **kwargs):
        '''
        Download a File
        https://api.doorloop.com/reference/download-file
        '''
        response = self.connector.get(f'{self.endpoint_base}/{file_id}/download', **kwargs)
        return self.validator(response).validate()



from .base import DoorLoopBase


class DoorLoopTask(DoorLoopBase):
    endpoint_base = '/tasks'

    def list(self, **kwargs):
        '''
        Retrieve all Tasks
        https://api.doorloop.com/reference/get-tasks
        '''
        response = self.connector.get(f'{self.endpoint_base}', **kwargs)
        return self.validator(response).validate()

    def create(self, data, **kwargs):
        '''
        Create a Task
        https://api.doorloop.com/reference/post-task
        '''
        response = self.connector.post(f'{self.endpoint_base}', data, **kwargs)
        return self.validator(response).validate()

    def post_update(self, data, **kwargs):
        '''
        Post an Update on a Task
        https://api.doorloop.com/reference/post-task-update
        '''
        response = self.connector.post(f'{self.endpoint_base}/update', data, **kwargs)
        return self.validator(response).validate()

    def retrieve(self, task_id, **kwargs):
        '''
        Retrieve a Task
        https://api.doorloop.com/reference/get-task
        '''
        response = self.connector.get(f'{self.endpoint_base}/{task_id}', **kwargs)
        return self.validator(response).validate()

    def update(self, task_id, data, **kwargs):
        '''
        Update a Task
        https://api.doorloop.com/reference/put-task
        '''
        response = self.connector.put(f'{self.endpoint_base}/{task_id}', data, **kwargs)
        return self.validator(response).validate()

    def delete(self, task_id, **kwargs):
        '''
        Deletes a Task
        https://api.doorloop.com/reference/delete-task
        '''
        response = self.connector.delete(f'{self.endpoint_base}/{task_id}', **kwargs)
        return self.validator(response).validate()


from .base import DoorLoopBase


class DoorLoopLease(DoorLoopBase):
    endpoint_base = '/leases'

    def list(self, **kwargs):
        '''
        List all Leases
        https://api.doorloop.com/reference/get-leases
        '''
        response = self.connector.get(f'{self.endpoint_base}', **kwargs)
        return self.validator(response).validate()

    def retrieve(self, lease_id, **kwargs):
        '''
        Retrieve a Lease
        https://api.doorloop.com/reference/get-lease
        '''
        response = self.connector.get(f'{self.endpoint_base}/{lease_id}', **kwargs)
        return self.validator(response).validate()

    def move_in_tenant(self, data, **kwargs):
        '''
        Move in Tenant
        https://api.doorloop.com/reference/post-leases-move-in
        '''
        response = self.connector.post(f'{self.endpoint_base}/move-in', data, **kwargs)
        return self.validator(response).validate()

    def move_out_tenant(self, data, **kwargs):
        '''
        Move out Tenant
        https://api.doorloop.com/reference/post-leases-move-out
        '''
        response = self.connector.post(f'{self.endpoint_base}/move-out', data, **kwargs)
        return self.validator(response).validate()

    def list_lease_tenants(self, *kwargs):
        '''
        List all Lease Tenants
        https://api.doorloop.com/reference/get-lease-tenants
        '''
        response = self.connector.get(f'{self.endpoint_base}/lease-tenants', **kwargs)
        return self.validator(response).validate()





import os

from .base import DoorLoopConnector, DoorLoopValidator

from .accounts import DoorLoopAccount
from .users import DoorLoopUser
from .properties import DoorLoopProperty
from .units import DoorLoopUnit
from .leases import DoorLoopLease
from .tenants import DoorLoopTenant
from .lease_payments import DoorLoopLeasePayment
from .lease_returned_payments import DoorLoopLeaseReturnedPayments
from .lease_charges import DoorLoopLeaseCharge
from .lease_credits import DoorLoopLeaseCredit
from .portfolios import DoorLoopPortfolio
from .tasks import DoorLoopTask
from .owners import DoorLoopOwner
from .vendors import DoorLoopVendor
from .expenses import DoorLoopExpense
from .vendor_bills import DoorLoopVendorBill
from .vendor_credits import DoorLoopVendorCredit
from .reports import DoorLoopReport
from .communications import DoorLoopCommunication
from .notes import DoorLoopNote
from .files import DoorLoopFile


class DoorLoop:

    def __init__(self, api_key=None, connector=None, validator=None):
        self.api_key = api_key or os.environ.get('DOORLOOP_API_KEY')
        self.connector = connector or DoorLoopConnector(api_key=api_key)
        self.validator = validator or DoorLoopValidator()

    @property
    def accounts(self):
        return DoorLoopAccount(self.connector, self.validator)

    @property
    def users(self):
        return DoorLoopUser(self.connector, self.validator)

    @property
    def properties(self):
        return DoorLoopProperty(self.connector, self.validator)

    @property
    def units(self):
        return DoorLoopUnit(self.connector, self.validator)

    @property
    def leases(self):
        return DoorLoopLease(self.connector, self.validator)

    @property
    def tenants(self):
        return DoorLoopTenant(self.connector, self.validator)

    @property
    def lease_payments(self):
        return DoorLoopLeasePayment(self.connector, self.validator)

    @property
    def lease_returned_payments(self):
        return DoorLoopLeaseReturnedPayments(self.connector, self.validator)

    @property
    def lease_charges(self):
        return DoorLoopLeaseCharge(self.connector, self.validator)

    @property
    def lease_credits(self):
        return DoorLoopLeaseCredit(self.connector, self.validator)

    @property
    def portfolios(self):
        return DoorLoopPortfolio(self.connector, self.validator)

    @property
    def tasks(self):
        return DoorLoopTask(self.connector, self.validator)

    @property
    def owners(self):
        return DoorLoopOwner(self.connector, self.validator)

    @property
    def vendors(self):
        return DoorLoopVendor(self.connector, self.validator)

    @property
    def expenses(self):
        return DoorLoopExpense(self.connector, self.validator)

    @property
    def vendor_bills(self):
        return DoorLoopVendorBill(self.connector, self.validator)

    @property
    def vendor_credits(self):
        return DoorLoopVendorCredit(self.connector, self.validator)

    @property
    def reports(self):
        return DoorLoopReport(self.connector, self.validator)

    @property
    def communications(self):
        return DoorLoopCommunication(self.connector, self.validator)

    @property
    def notes(self):
        return DoorLoopNote(self.connector, self.validator)

    @property
    def files(self):
        return DoorLoopFile(self.connector, self.validator)



from .base import DoorLoopBase


class DoorLoopLeaseReturnedPayments(DoorLoopBase):
    endpoint_base = '/lease-reversed-payments'

    def list(self, **kwargs):
        '''
        List all Lease Returned Payments
        https://api.doorloop.com/reference/get-lease-reversed-payments
        '''
        response = self.connector.get(f'{self.endpoint_base}', **kwargs)
        return self.validator(response).validate()

    def create(self, data, **kwargs):
        '''
        Create a Lease Returned Payment
        https://api.doorloop.com/reference/post-lease-reversed-payments
        '''
        response = self.connector.post(f'{self.endpoint_base}', data, **kwargs)
        return self.validator(response).validate()

    def retrieve(self, lease_returned_payment_id, **kwargs):
        '''
        Retrieve a Lease Returned Payment
        https://api.doorloop.com/reference/get-lease-reversed-payment
        '''
        response = self.connector.get(f'{self.endpoint_base}/{lease_returned_payment_id}', **kwargs)
        return self.validator(response).validate()

    def delete(self, lease_returned_payment_id, **kwargs):
        '''
        Delete a Lease Returned Payment
        https://api.doorloop.com/reference/delete-lease-reversed-payment
        '''
        response = self.connector.delete(f'{self.endpoint_base}/{lease_returned_payment_id}', **kwargs)
        return self.validator(response).validate()



from .base import DoorLoopBase


class DoorLoopProperty(DoorLoopBase):
    endpoint_base = '/properties'

    def list(self, **kwargs):
        '''
        List all Properties
        https://api.doorloop.com/reference/get-properties
        '''
        response = self.connector.get(f'{self.endpoint_base}', **kwargs)
        return self.validator(response).validate()

    def retrieve(self, property_id, **kwargs):
        '''
        Retrieve a Property
        https://api.doorloop.com/reference/get-property
        '''
        response = self.connector.get(f'{self.endpoint_base}/{property_id}', **kwargs)
        return self.validator(response).validate()



from .base import DoorLoopBase


class DoorLoopReport(DoorLoopBase):
    endpoint_base = '/reports'

    def rent_roll(self, **kwargs):
        '''
        Rent Roll
        https://api.doorloop.com/reference/get-reports-rent-roll
        '''
        response = self.connector.get(f'{self.endpoint_base}/rent-roll', **kwargs)
        return self.validator(response).validate()

    def profit_and_loss(self, **kwargs):
        '''
        Profit & Loss
        https://api.doorloop.com/reference/get-reports-profit-and-loss
        '''
        response = self.connector.get(f'{self.endpoint_base}/profit-and-loss-summary', **kwargs)
        return self.validator(response).validate()

    def cash_flow_statement(self, **kwargs):
        '''
        Cash Flow Statement
        https://api.doorloop.com/reference/get-reports-cash-flow-statement
        '''
        response = self.connector.get(f'{self.endpoint_base}/cash-flow-statement', **kwargs)
        return self.validator(response).validate()

    def balance_sheet(self, **kwargs):
        '''
        Balance Sheet
        https://api.doorloop.com/reference/get-reports-balance-sheet
        '''
        response = self.connector.get(f'{self.endpoint_base}/balance-sheet-summary', **kwargs)
        return self.validator(response).validate()



from .base import DoorLoopBase


class DoorLoopTenant(DoorLoopBase):
    endpoint_base = '/tenants'

    def retrieve(self, tenant_id, **kwargs):
        '''
        Retrieve a Tenant
        https://api.doorloop.com/reference/get-tenant
        '''
        response = self.connector.get(f'{self.endpoint_base}/{tenant_id}', **kwargs)
        return self.validator(response).validate()

    def update(self, tenant_id, data, **kwargs):
        '''
        Update a Tenant
        https://api.doorloop.com/reference/put-tenant
        '''
        response = self.connector.put(f'{self.endpoint_base}/{tenant_id}', data, **kwargs)
        return self.validator(response).validate()

    def delete(self, tenant_id, **kwargs):
        '''
        Delete a Tenant
        https://api.doorloop.com/reference/delete-tenant
        '''
        response = self.connector.delete(f'{self.endpoint_base}/{tenant_id}', **kwargs)
        return self.validator(response).validate()

    def list(self, **kwargs):
        '''
        Retrieve all Tenants
        https://api.doorloop.com/reference/get-tenants
        '''
        response = self.connector.get(f'{self.endpoint_base}', **kwargs)
        return self.validator(response).validate()

    def create_prospect(self, data, **kwargs):
        '''
        Create a Prospect
        https://api.doorloop.com/reference/post-tenant
        '''
        response = self.connector.post(f'{self.endpoint_base}', data, **kwargs)
        return self.validator(response).validate()



from .base import DoorLoopBase


class DoorLoopUser(DoorLoopBase):
    endpoint_base = '/users'

    def list(self, **kwargs):
        '''
        List all Users
        https://api.doorloop.com/reference/get-users
        '''
        response = self.connector.get(f'{self.endpoint_base}', **kwargs)
        return self.validator(response).validate()

    def retrieve(self, user_id, **kwargs):
        '''
        Retrieve a User
        https://api.doorloop.com/reference/get-user
        '''
        response = self.connector.get(
            f'{self.endpoint_base}/{user_id}',
            **kwargs
        )
        return self.validator(response).validate()

    def get_current(self, **kwargs):
        '''
        Get Current User
        https://api.doorloop.com/reference/get-current-user
        '''
        response = self.connector.get(f'{self.endpoint_base}/me', **kwargs)
        return self.validator(response).validate()



from .base import DoorLoopBase


class DoorLoopExpense(DoorLoopBase):
    endpoint_base = '/expenses'

    def list(self, **kwargs):
        '''
        List all Expenses
        https://api.doorloop.com/reference/get-expenses
        '''
        response = self.connector.get(f'{self.endpoint_base}', **kwargs)
        return self.validator(response).validate()

    def create(self, data, **kwargs):
        '''
        Create an Expense
        https://api.doorloop.com/reference/post-expense
        '''
        response = self.connector.post(f'{self.endpoint_base}', data, **kwargs)
        return self.validator(response).validate()

    def retrieve(self, expense_id, **kwargs):
        '''
        Retrieve an Expense
        https://api.doorloop.com/reference/get-expense
        '''
        response = self.connector.get(f'{self.endpoint_base}/{expense_id}', **kwargs)
        return self.validator(response).validate()

    def update(self, expense_id, data, **kwargs):
        '''
        Update an Expense
        https://api.doorloop.com/reference/put-expense
        '''
        response = self.connector.put(f'{self.endpoint_base}/{expense_id}', data, **kwargs)
        return self.validator(response).validate()

    def delete(self, expense_id, **kwargs):
        '''
        Delete an Expense
        https://api.doorloop.com/reference/delete-expense
        '''
        response = self.connector.delete(f'{self.endpoint_base}/{expense_id}', **kwargs)
        return self.validator(response).validate()


from .base import DoorLoopBase


class DoorLoopVendorCredit(DoorLoopBase):
    endpoint_base = '/vendor-credits'
    
    def list(self, **kwargs):
        '''
        List all Vendor Credits
        https://api.doorloop.com/reference/get-vendor-credits
        '''
        response = self.connector.get(f'{self.endpoint_base}', **kwargs)
        return self.validator(response).validate()

    def create(self, data, **kwargs):
        '''
        Create a Vendor Credit
        https://api.doorloop.com/reference/post-vendor-credit
        '''
        response = self.connector.post(f'{self.endpoint_base}', data, **kwargs)
        return self.validator(response).validate()

    def retrieve(self, vendor_credit_id, **kwargs):
        '''
        Retrieve a Vendor Credit
        https://api.doorloop.com/reference/get-vendor-credit
        '''
        response = self.connector.get(f'{self.endpoint_base}/{vendor_credit_id}', **kwargs)
        return self.validator(response).validate()

    def update(self, vendor_credit_id, data, **kwargs):
        '''
        Update a Vendor Credit
        https://api.doorloop.com/reference/put-vendor-credit
        '''
        response = self.connector.put(f'{self.endpoint_base}/{vendor_credit_id}', data, **kwargs)
        return self.validator(response).validate()

    def delete(self, vendor_credit_id, **kwargs):
        '''
        Delete a Vendor Credit
        https://api.doorloop.com/reference/delete-vendor-credit
        '''
        response = self.connector.delete(f'{self.endpoint_base}/{vendor_credit_id}', **kwargs)
        return self.validator(response).validate()



from .base import DoorLoopBase


class DoorLoopUnit(DoorLoopBase):
    endpoint_base = '/units'

    def list(self, **kwargs):
        '''
        List all Units
        https://api.doorloop.com/reference/get-units
        '''
        response = self.connector.get(f'{self.endpoint_base}', **kwargs)
        return self.validator(response).validate()

    def retrieve(self, unit_id, **kwargs):
        '''
        Retrieve a Unit
        https://api.doorloop.com/reference/get-unit
        '''
        response = self.connector.get(f'{self.endpoint_base}/{unit_id}', **kwargs)
        return self.validator(response).validate()
